#ifndef __SERVER_H
#define __SERVER_H

/* defined in server_dslconf_print.c */
void dslconf_print_init(struct interface *, int);
void dslconf_print(int, const char *);
extern int dslconf_print_loglevel;

#define dprint(severity, ...) if ((severity) >= dslconf_print_loglevel) { \
	char __printv_message[1024]; \
	snprintf(__printv_message, 1024, "%s, %s(): ", __FILE__, __func__); \
	snprintf(__printv_message + strlen(__printv_message), \
			1024 - strlen(__printv_message), __VA_ARGS__); \
	fprintf(stderr, "%s\n", __printv_message); \
	dslconf_print((severity), __printv_message); \
}
#define dnotify(...)	dprint(REQ_NOTIFY, __VA_ARGS__)
#define dwarn(...)	dprint(REQ_WARNING, __VA_ARGS__)
#define derror(...)	dprint(REQ_ERROR, __VA_ARGS__)

/* mimic kernel's ARRAY_SIZE macro */
#define ARRAY_SIZE(x) (sizeof(x) / sizeof(*(x)))

/* simple command: no update or notify routines */
#define SIMPLE_DSLCONF_CMD(t, c)		\
struct dslconf_cmd_s CMD_##t = {		\
	.cb = {					\
		.type = REQ_##t,		\
		.callback = &c##_callback,	\
	},					\
	.update = NULL,				\
	.notify = NULL,				\
}

/* regular command: callback, update and notify must exist */
#define DSLCONF_CMD(t, c)			\
struct dslconf_cmd_s CMD_##t = {		\
	.cb = {					\
		.type = REQ_##t,		\
		.callback = &c##_callback,	\
	},					\
	.update = &c##_update,			\
	.notify = &c##_notify,			\
}

struct dslconf_cmd_s {
	/* listener_callback to pass on to dslconf listener() routine */
	struct listener_callback cb;
	/* update(): called once per second */
	int (*update)(void);
	/* notify(): called if update() returned > 0 */
	void (*notify)(struct interface iface);
};

/* forward declarations for the structs defined in server_cmd_*.c */
struct dslconf_cmd_s CMD_NOP;
struct dslconf_cmd_s CMD_EXEC;
struct dslconf_cmd_s CMD_GET_ENV;
struct dslconf_cmd_s CMD_SET_ENV;
#ifdef TARGET_AR7
struct dslconf_cmd_s CMD_FLASH_CFG;
struct dslconf_cmd_s CMD_START_LOGIC;
struct dslconf_cmd_s CMD_RELOAD_LOGIC;
struct dslconf_cmd_s CMD_STOP_LOGIC;
struct dslconf_cmd_s CMD_FLASH_FW;
#endif
struct dslconf_cmd_s CMD_REEXEC;
struct dslconf_cmd_s CMD_GET_GW;
struct dslconf_cmd_s CMD_GET_IP;
struct dslconf_cmd_s CMD_SET_IP;
struct dslconf_cmd_s CMD_GET_PUB_IP;
struct dslconf_cmd_s CMD_SYNC_STAT;
struct dslconf_cmd_s CMD_RATE_INFO;
struct dslconf_cmd_s CMD_PPP_INFO;
struct dslconf_cmd_s CMD_GET_VERSION;
struct dslconf_cmd_s CMD_FS_STAMP;
struct dslconf_cmd_s CMD_SCMVER;
struct dslconf_cmd_s CMD_GET_PPPD_LOG;
#ifdef TARGET_FUSIV
struct dslconf_cmd_s CMD_SET_VDSL_CFG;
struct dslconf_cmd_s CMD_USE_BUILTIN_VDSL_CFG;
struct dslconf_cmd_s CMD_WAN_MGR;
struct dslconf_cmd_s CMD_VDSL_CFG_SHA1;
struct dslconf_cmd_s CMD_VDSL_MON_STATS;
#endif

/* file containing 'FSSTAMP=<datestring>'
 * and 'SCMVER=<git-commit-id>' */
#define FSSTAMP_FILE "/etc/versions"

/* path to PPP daemon's log file */
#if defined(TARGET_FUSIV)
#  define PPPD_LOGFILE "/tmp/pppd.log"
#else /* defined(TARGET_AR7) */
#  define PPPD_LOGFILE "/var/pppd.log"
#endif

/* functions defined in server_cmd_common.c */
char *read_fp_to_buf(FILE *);
char *get_cmd_output(const char *);
int read_fsstamp_file(void);
char *fsstamp, *scmver;

#endif /* __SERVER_H */
