/* vwmfdiag.c	Tue Jun 24 2009 09:46:42  sekang */

/*

Module:	vwmfdiag.c

Function:
	Driver for Vertex Wireless diagnostic port

Version:
	V1.00a	Tue Jun 24 2009 09:46:42 sekang	Edit level 1

Copyright notice:
	This file copyright (C) 2009 by

		MCCI Corporation
		3520 Krums Corners Road
		Ithaca, NY 14850

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License	
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 
Author:
	Seongeun Kang, MCCI Corporation	June 2009

Revision history:
   1.00a Thu Jun 24 2009 sekang
	Module created.

*/

#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/tty.h>
#include <linux/tty_driver.h>
#include <linux/tty_flip.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/usb.h>
#include <asm/byteorder.h>
#include <asm/unaligned.h>
#include <asm/uaccess.h>
#include <linux/list.h>

//#include "usb-serial.h"
#include <linux/usb/serial.h>

static int __init vwmfdiag_register(void);
static void __exit vwmfdiag_deregister(void);
void vwmfdiag_shutdown(struct usb_serial *serial);

static int debug;
module_param(debug, bool, S_IRUGO | S_IWUSR);
MODULE_PARM_DESC(debug, "Debug enabled or not");
#define MCCI_DEBUG(a)   if (debug) {(a);}

#define VERTEX_VENDOR_ID	0x1FE7
#define VERTEX_PRODUCT_ID	0x100

#define USB_VENDOR_INTERFACE(vend,prod,ifcclass) \
	.match_flags = USB_DEVICE_ID_MATCH_DEVICE | USB_DEVICE_ID_MATCH_INT_CLASS, \
	.idVendor = (vend), \
	.idProduct = (prod), \
	.bInterfaceClass = (ifcclass)


static struct usb_device_id vwmfdiag_id_table[] =
	{
		{
		USB_VENDOR_INTERFACE(VERTEX_VENDOR_ID, VERTEX_PRODUCT_ID, 255)
		},
		{
		/* EMPTY */
		}
	};

MODULE_DEVICE_TABLE (usb, vwmfdiag_id_table);

/* we want to look at all devices, as the vendor/product id can change
 * depending on the command line argument */
//static struct usb_device_id vwmfdiag_serial_ids[] = {
//	{.driver_info = 42},
//	{}
//};

// SERIAL INFO
struct usb_serial_driver vwmfdiag_device =
	{
	.driver =
		{
		.owner =	THIS_MODULE,
		.name =		"vwmfdiag",
		},
	.id_table =		vwmfdiag_id_table,
	.num_ports =		1,
#if 0
	.shutdown =		vwmfdiag_shutdown,
#else
	.release =		vwmfdiag_shutdown,
#endif
	};

static struct usb_serial_driver * const serial_drivers[] = {
	&vwmfdiag_device,
	NULL,
};

void vwmfdiag_shutdown(struct usb_serial *serial)
	{
#if 0
	struct usb_serial_port *thisport;
        int i;
#endif

	MCCI_DEBUG(printk(KERN_INFO "vwmfdiag_shutdown: Entering, num ports %d\n",
		serial->num_ports));
#if 0
        /* stop reads and writes on all ports */
        for (i=0; i < serial->num_ports; ++i)
		{
		thisport = serial->port[i];

		if (serial->dev) {
			/* shutdown any bulk reads that might be going on */
			if (serial->num_bulk_out)
				usb_kill_urb(thisport->write_urb);
			if (serial->num_bulk_in)
				usb_kill_urb(thisport->read_urb);
			}
		}
#endif
	MCCI_DEBUG(printk(KERN_INFO "vwmfdiag_shutdown: Returning\n"));
	}

MODULE_AUTHOR("MCCI");
MODULE_DESCRIPTION("Vertex Wireless Diagnostic Port Driver");
MODULE_LICENSE("GPL");

module_usb_serial_driver(serial_drivers, vwmfdiag_id_table);
